@@ -3,10 +3,11 @@ package EBook::MOBI;
 use strict;
 use warnings;
 
-our $VERSION = 0.62;
+our $VERSION = 0.68;
 
 # needed CPAN stuff
 use File::Temp qw(tempfile);
+use Carp;
 
 # needed local stuff
 use EBook::MOBI::Driver::POD;
@@ -103,8 +104,36 @@ sub set_filename {
 
 sub set_encoding {
     my $self = shift;
+    my $encoding = shift;
 
-    $self->{encoding} = shift;
+    if (! $self->_encoding_to_codepage($encoding))
+    {
+        croak "Encoding $encoding is not supported by EPub::MOBI";
+    }
+
+    $self->{encoding} = $encoding;
+}
+
+sub _encoding_to_codepage
+{
+    # Translate a Perl encoding name into a codepage value or return
+    # undef if not supported.
+
+    my $self = shift;
+    my $encoding = shift;
+
+    if ($encoding =~ /iso-8859-1|ascii/i)
+    {
+        return 1252;
+    }
+    elsif ($encoding =~ /utf-*8/i)
+    {
+        return 65001;
+    }
+    else
+    {
+        return undef;
+    }
 }
 
 sub add_mhtml_content {
@@ -140,28 +169,7 @@ sub add_content {
         $parser->set_options($driver_opt);
     }
 
-    # ok, now we prepare the parsing, unfortunately we have to do
-    # some complicated magic with the string data...
-
-    # INPUT:
-    # We do this trick so that we have UTF8
-    # It seems like this is working after all...
-    my ($fh,$f_name) = tempfile();
-    binmode $fh, $self->{encoding};
-    print $fh $data;
-    close $fh;
-    open my $data_handle, "<$self->{encoding}", $f_name;
-
-    # and we have a file again...
-    my $input = '';
-    while (my $line = <$data_handle>) {
-        $input .= $line;
-    }
-    close $data_handle;
-    unlink $f_name;
-
-    # we call the parser to parse, result will be in $buffer4html
-    my $output = $parser->parse($input);
+    my $output = $parser->parse($data);
 
     $self->{html_data} .= $output;
 }
@@ -216,10 +224,13 @@ sub save {
     $mobi->debug_on($self->{ref_to_debug_sub})
         if ($self->{ref_to_debug_sub});
 
+    my $codepage = $self->_encoding_to_codepage($self->{encoding});
+
     $mobi->pack(    $self->{html_data},
                     $self->{filename},
                     $self->{author},
                     $self->{title},
+                    $codepage
                 );
 }
 
@@ -301,6 +312,8 @@ EBook::MOBI - create an ebook in the MOBI format.
 
 You are at the right place here if you want to create an ebook in the so called MOBI format (somethimes also called PRC format or Mobipocket).
 
+This is a software library for the I<perl programming language>.
+
 =head1 SYNOPSIS
 
 If you plan to create a typical ebook you probably will need most of the methods provided by this module. So it might be a good idea to read all the descriptions in the methods section, and also have a look at this example here.
@@ -409,10 +422,13 @@ If you don't use this method, the default name will be 'book.mobi'.
 =head2 set_encoding
 
 If you don't set anything here, C<:encoding(UTF-8)> will be default.
-As far as I know, only CP1252 (Win Latin1) und UTF-8 are supported by popular readers.
 
  $book->set_encoding(':encoding(UTF-8)');
 
+Only two encodings are supported by the mobiperl driver: UTF-8 or
+ISO-8859-1. ASCII is treated the same as ISO-8859-1. If any other
+encodings are passed in, the module will raise an error.
+
 Please see L<http://perldoc.perl.org/functions/binmode.html> for the syntax of your encoding keyword.
 If you use use hardcoded strings in your program, C<use utf8;> should be helping.
 
@@ -433,6 +449,12 @@ If you indent the 'h1' tag with any whitespace, it will not appear in the TOC (o
 
 There is a module L<EBook::MOBI::Converter> which helps you in creating this format. See it's documentation for more information.
 
+If you are passing your own text to add_mhtml_content rather than
+using a converter you will need to: a) encode the text according to
+your chosen encoding, eg call L<Encode::encode_utf8>; b) ensure that
+any HTML entities such as '<' in your text are replaced, eg by calling
+L<HTML::Entities::encode_entities>.
+
 =head2 add_content
 
 Use this method if you have your content in a specific markup format.
@@ -601,9 +623,11 @@ For adding manually in MHTML, see L<EBook::MOBI::Converter>.
 
 =over
 
-=item * L<GARU|https://metacpan.org/author/GARU>
+=item * L<Breno G. de Oliveira (GARU)|https://metacpan.org/author/GARU>
 
-=item * L<RENEEB|https://metacpan.org/author/RENEEB>
+=item * L<Renée Bäcker (RENEEB)|https://metacpan.org/author/RENEEB>
+
+=item * L<Rupert Lane (RUPERTL)|https://metacpan.org/author/RUPERTL>
 
 =back
 
@@ -616,7 +640,7 @@ This program is free software; you can redistribute it and/or modify it under th
 
 =head1 AUTHOR
 
-Boris Däppen E<lt>boris_daeppen@bluewin.chE<gt>
+Boris Däppen E<lt>bdaeppen.perl@gmail.comE<gt>
 
 =cut
 
